# -*- coding: utf-8 -*-
"""WebHelpers used in userprofile."""


def user_avatar(user):
    if user is None:
        return None

    from .lib import get_user_data
    _, avatar = get_user_data(user)
    return avatar


# -*- coding: utf-8 -*-
import tg
from formencode import FancyValidator
from tgext.pluggable import app_model
from userprofile.model import DBSession

try:
    from tgext.datahelpers.validators import SQLAEntityConverter as _SQLAEntityConverter
except ImportError:
    class _SQLAEntityConverter(object):
        def __init__(self, *args, **kwargs): pass

try:
    from tgext.datahelpers.validators import MingEntityConverter as _MingEntityConverter
except ImportError:
    class _MingEntityConverter(object):
        def __init__(self, *args, **kwargs): pass


def _get_storage_backend():
    tg_conf = tg.config
    if tg_conf.get('use_sqlalchemy', False):
        return 'sqlalchemy'
    elif tg_conf.get('use_ming', False):
        return 'ming'
    else:
        raise ValueError('Turbopress should be used with sqlalchemy or ming')


class ModelEntityConverter(FancyValidator):
    def __init__(self, klass, slugified=False):
        super(FancyValidator, self).__init__(not_empty=True)
        self.model_name = klass
        self.slugified = slugified
        self._converters = {}

    def _create_converter(self, backend):
        if backend == 'sqlalchemy':
            return _SQLAEntityConverter(getattr(app_model, self.model_name),
                                        session=DBSession,
                                        slugified=self.slugified)
        elif backend == 'ming':
            return _MingEntityConverter(getattr(app_model, self.model_name),
                                        slugified=self.slugified)

    @property
    def _converter(self):
        storage_backend = _get_storage_backend()
        try:
            converter = self._converters[storage_backend]
        except KeyError:
            self._converters[storage_backend] = converter = self._create_converter(storage_backend)
        return converter

    def _convert_to_python(self, value, state):
        return self._converter._to_python(value, state)

    def _convert_from_python(self, value, state):
        return self._converter._from_python(value, state)

    def _validate_python(self, value, state):
        return self._converter.validate_python(value, state)

    _to_python = _convert_to_python
    _from_python = _convert_from_python
    validate_python = _validate_python


# -*- coding: utf-8 -*-
from formencode import validators
from hashlib import md5
import tg
from tg import url
from tgext.pluggable import app_model, plug_url


def get_profile_css(config):
    return url(config['_pluggable_userprofile_config'].get('custom_css',
        '/_pluggable/userprofile/css/style.css'))


def _get_user_gravatar(email_address):
    mhash = md5(email_address).hexdigest()
    return url('http://www.gravatar.com/avatar/'+mhash, params=dict(s=32))


def get_user_data(user):
    user_data = getattr(user, 'profile_data', {'display_name':('Display Name', user.display_name),
                                               'email_address':('Email Address', user.email_address)})

    user_avatar = user_data.pop('avatar', None)
    if user_avatar is None:
        fbauth_info = getattr(user, 'fbauth', None)
        if fbauth_info is not None:
            user_avatar = fbauth_info.profile_picture + '?type=large'
        else:
            user_avatar = _get_user_gravatar(user_data['email_address'][1])

    return user_data, user_avatar

def update_user_data(user, user_data):
    for k, v in user_data.items():
        setattr(user, k, v)

if tg.config.get('prefer_toscawidgets2', False):
    from tw2.forms import ListForm, TextField, TextArea, HiddenField, FileField, SubmitButton, PasswordField
else:
    from tw.forms import ListForm, TextField, PasswordField

from formencode.validators import UnicodeString, FieldStorageUploadConverter
from sprox.formbase import FilteringSchema
from formencode.validators import FieldsMatch

_password_match = FieldsMatch('password', 'verify_password',
                              messages={'invalidNoMatch': 'Passwords do not match'})

if hasattr(TextField, 'req'):
    change_password_form_validator = _password_match
else:
    change_password_form_validator =  FilteringSchema(chained_validators=[_password_match])

if tg.config.get('prefer_toscawidgets2', False):
    from tw2.core import Required

    class UserForm(ListForm):
        uid=HiddenField()
        submit=SubmitButton(value='Save')

    def create_user_form(user):
        profile_form = getattr(user, 'profile_form', None)
        if not profile_form:
            user_data, user_avatar = get_user_data(user)
            profile_form = UserForm()

            for name, info in user_data.items():
                profile_form.child = profile_form.child()
                profile_form.child.children.append(TextField(id=name, validator=Required, label=info[0]))

            profile_form = profile_form()
        return profile_form

    class ChangePasswordForm(ListForm):
        password = PasswordField(label=u'Password', validator=Required)
        verify_password = PasswordField(label=u'Confirm Password', validator=Required)
        submit=SubmitButton(value='Save')
        validator = change_password_form_validator

    def create_change_password_form():
        return ChangePasswordForm()
else:
    def create_user_form(user):
        profile_form = getattr(user, 'profile_form', None)
        if not profile_form:
            user_data, user_avatar = get_user_data(user)
            form_fields = [TextField(id=name, validator=UnicodeString(not_empty=True),
                                     label_text=info[0]) for name, info in user_data.items()]
            profile_form = ListForm(fields=form_fields, submit_text='Save',
                                    action=plug_url('userprofile', '/save'))
        return profile_form

    def create_change_password_form():
        return ListForm(fields=[PasswordField('password', label_text='Password',
                                              validator=UnicodeString(not_empty=True)),
                                PasswordField('verify_password', label_text='Confirm Password',
                                              validator=UnicodeString(not_empty=True))],
                        action=plug_url('userprofile', '/save_password', lazy=True),
                        validator=change_password_form_validator,
                        submit_text='Save')


# -*- coding: utf-8 -*-
"""The userprofile package"""

def plugme(app_config, options):
    app_config['_pluggable_userprofile_config'] = options
    return dict(appid='userprofile', global_helpers=False)



# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import TGController
from tg import expose, flash, require, url, lurl, request, redirect, validate, config
from tg.util import Bunch
from tg.i18n import ugettext as _, lazy_ugettext as l_

try:
    from repoze.what import predicates
except ImportError:
    from tg import predicates

from userprofile.lib import create_user_form, get_user_data, get_profile_css, \
                            update_user_data, create_change_password_form
from userprofile.lib.validators import ModelEntityConverter
from tgext.pluggable import app_model, plug_url, primary_key
from tgext.datahelpers.utils import fail_with


edit_password_form = create_change_password_form()


class RootController(TGController):
    @expose('userprofile.templates.index')
    @validate({'user': ModelEntityConverter('User')},
              error_handler=fail_with(404))
    def _default(self, user, **kw):
        user_data, user_avatar = get_user_data(user)
        user_displayname = user_data.pop('display_name', (None, 'Unknown'))
        user_partial = config['_pluggable_userprofile_config'].get('user_partial')
        return dict(user=user, is_my_own_profile=request.identity and request.identity['user'] == user,
                    user_data=user_data, user_avatar=user_avatar,
                    user_displayname=user_displayname,
                    profile_css=get_profile_css(config),
                    user_partial=user_partial)

    @expose('userprofile.templates.edit')
    @require(predicates.not_anonymous())
    def edit(self):
        user = request.identity['user']
        user_data, user_avatar = get_user_data(user)
        user_data = Bunch(((fieldid, info[1]) for fieldid, info in user_data.items()))
        return dict(user=user_data, profile_css=get_profile_css(config),
                    user_avatar=user_avatar,
                    form=create_user_form(user))

    @expose()
    @require(predicates.not_anonymous())
    def save(self, **kw):
        user = request.identity['user']
        profile_save = getattr(user, 'save_profile', None)
        if not profile_save:
            profile_save = update_user_data
        profile_save(user, kw)
        flash(_('Profile successfully updated'))
        return redirect(plug_url('userprofile', '/%s' % getattr(user, primary_key(app_model.User).name)))

    @expose('userprofile.templates.chpasswd')
    @require(predicates.not_anonymous())
    def chpasswd(self, **kw):
        return dict(profile_css=get_profile_css(config),
                    form=edit_password_form)

    @require(predicates.not_anonymous())
    @expose()
    @validate(edit_password_form, error_handler=chpasswd)
    def save_password(self, password, verify_password):
        user = request.identity['user']
        user.password = password
        flash(_('Password successfully changed'))
        return redirect(plug_url('userprofile', '/%s' % getattr(user, primary_key(app_model.User).name)))

    @require(predicates.not_anonymous())
    @expose('userprofile.templates.index')
    def me(self):
        user=request.identity['user']
        return redirect(plug_url('userprofile', '/%s' % getattr(user, primary_key(app_model.User).name)))